const { declare } = require('@babel/helper-plugin-utils');

const defaultTargets = '> 1%, last 2 versions, Firefox ESR, not IE > 0, not IE_Mob > 0, not dead';

const getPresets = (modules, targets, react, development, flow, minify) => {
  const presets = [
    [
      '@babel/preset-env',
      {
        modules: modules === false ? false : 'auto',
        targets,
      },
    ],
  ];

  if (react) {
    presets.push(['@babel/preset-react', { development }]);
  }

  if (flow) {
    presets.push('@babel/preset-flow');
  }

  if (minify) {
    presets.push('babel-preset-minify');
  }

  return presets;
};

const getPlugins = (development, stage, react, node, minify) => {
  let plugins = [];
  if (development) {
    plugins = plugins.concat(['babel-plugin-console-source']);
  }

  plugins = plugins.concat(['babel-plugin-fast-async']);

  if (stage === 0) {
    plugins = plugins.concat(['@babel/plugin-proposal-function-bind']);
  }

  if (stage <= 1) {
    plugins = plugins.concat([
      '@babel/plugin-proposal-export-default-from',
      '@babel/plugin-proposal-logical-assignment-operators',
      ['@babel/plugin-proposal-optional-chaining', { loose: false }],
      ['@babel/plugin-proposal-pipeline-operator', { proposal: 'minimal' }],
      ['@babel/plugin-proposal-nullish-coalescing-operator', { loose: false }],
      '@babel/plugin-proposal-do-expressions',
    ]);
  }

  if (stage <= 2) {
    plugins = plugins.concat([
      ['@babel/plugin-proposal-decorators', { legacy: true }],
      '@babel/plugin-proposal-function-sent',
      '@babel/plugin-proposal-export-namespace-from',
      '@babel/plugin-proposal-numeric-separator',
      '@babel/plugin-proposal-throw-expressions',
    ]);
  }

  if (stage <= 3) {
    plugins = plugins.concat([
      '@babel/plugin-syntax-dynamic-import',
      '@babel/plugin-syntax-import-meta',
      ['@babel/plugin-proposal-class-properties', { loose: false }],
      '@babel/plugin-proposal-json-strings',
    ]);
  }

  plugins = plugins.concat([
    [
      '@babel/plugin-proposal-object-rest-spread',
      {
        useBuiltIns: true,
      },
    ],
    '@babel/plugin-transform-property-mutators',
    '@babel/plugin-transform-member-expression-literals',
    '@babel/plugin-transform-property-literals',
    '@babel/plugin-transform-jscript',
    [
      '@babel/plugin-transform-template-literals',
      {
        spec: true,
      },
    ],
  ]);

  if (react) {
    plugins = plugins.concat(['babel-plugin-hoist-facc']);
  }

  if (node) {
    plugins = plugins.concat(['babel-plugin-dynamic-import-node']);
  }

  if (minify && react && !development) {
    plugins = plugins.concat([
      '@babel/plugin-transform-react-constant-elements',
      '@babel/plugin-transform-react-inline-elements',
      'babel-plugin-transform-react-remove-prop-types',
    ]);
  }

  if (minify && node && !development) {
    plugins = plugins.concat(['babel-plugin-transform-node-env-inline']);
  }

  return plugins;
};

module.exports = declare((api, options) => {
  // see docs about api at https://babeljs.io/docs/en/config-files#apicache
  api.assertVersion('^7.1.2');

  const { modules, targets = options.targets || defaultTargets, stage = 3 } = options;

  if (typeof modules !== 'undefined' && typeof modules !== 'boolean' && modules !== 'auto') {
    throw new TypeError(
      'babel-preset-yarus only accepts `true`, `false`, or `"auto"` as the value of the "modules" option',
    );
  }

  if (!Number.isInteger(stage) && stage < 0 && stage > 4) {
    throw new TypeError(
      'babel-preset-yarus only accepts number from 0 to 4 as the value of the "stage" option',
    );
  }

  if (stage < 2) {
    process.emitWarning('Stage-0 and Stage-1 is dangerous! Don`t use it in production!');
  }

  const react = typeof options.react === 'boolean' ? options.react : false;
  const flow = typeof options.flow === 'boolean' ? options.flow : false;
  // prettier-ignore
  const development = typeof options.development === 'boolean'
    ? options.development
    : api.cache.using(() => process.env.NODE_ENV === 'development');
  const node = typeof options.node === 'boolean' ? options.node : false;
  const minify = typeof options.minify === 'boolean' ? options.minify : false;

  return {
    presets: getPresets(modules, targets, react, development, flow, minify),
    plugins: getPlugins(development, stage, react, node, minify),
  };
});
